/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef __GTK_WVIEWPORT_H__
#define __GTK_WVIEWPORT_H__


#include <gdk/gdk.h>
#include <gtk/gtkadjustment.h>
#include <gtk/gtkbin.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GTK_WVIEWPORT(obj)          GTK_CHECK_CAST (obj, gtk_wviewport_get_type (), GtkWViewport)
#define GTK_WVIEWPORT_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_wviewport_get_type (), GtkWViewportClass)
#define GTK_IS_WVIEWPORT(obj)       GTK_CHECK_TYPE (obj, gtk_wviewport_get_type ())


typedef struct _GtkWViewport       GtkWViewport;
typedef struct _GtkWViewportClass  GtkWViewportClass;

struct _GtkWViewport
{
  GtkBin bin;

  gint shadow_type;
  GdkWindow *view_window;
  GdkWindow *bin_window;
  GtkAdjustment *hadjustment;
  GtkAdjustment *vadjustment;
};

struct _GtkWViewportClass
{
  GtkBinClass parent_class;
};


guint          gtk_wviewport_get_type        (void);
GtkWidget*     gtk_wviewport_new             (GtkAdjustment *hadjustment,
					     GtkAdjustment *vadjustment);
GtkAdjustment* gtk_wviewport_get_hadjustment (GtkWViewport   *wviewport);
GtkAdjustment* gtk_wviewport_get_vadjustment (GtkWViewport   *wviewport);
void           gtk_wviewport_set_hadjustment (GtkWViewport   *wviewport,
					     GtkAdjustment *adjustment);
void           gtk_wviewport_set_vadjustment (GtkWViewport   *wviewport,
					     GtkAdjustment *adjustment);
void           gtk_wviewport_set_shadow_type (GtkWViewport   *wviewport,
					     GtkShadowType  type);


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GTK_WVIEWPORT_H__ */
